@@ -14,10 +14,10 @@ namespace TRICEPS_NS {
 
 void BasicAggregator::handle(Table *table, AggregatorGadget *gadget, Index *index,
 	const IndexType *parentIndexType, GroupHandle *gh, Tray *dest,
-	AggOp aggop, Rowop::Opcode opcode, RowHandle *rh, Tray *copyTray)
+	AggOp aggop, Rowop::Opcode opcode, RowHandle *rh)
 {
 	const BasicAggregatorType *at = static_cast<const BasicAggregatorType *>(gadget->getType());
-	at->cb_(table, gadget, index, parentIndexType, gh, dest, aggop, opcode, rh, copyTray);
+	at->cb_(table, gadget, index, parentIndexType, gh, dest, aggop, opcode, rh);
 }
 
 }; // TRICEPS_NS
