@@ -26,7 +26,7 @@ public:
 	// from Aggregator
 	virtual void handle(Table *table, AggregatorGadget *gadget, Index *index,
 		const IndexType *parentIndexType, GroupHandle *gh, Tray *dest,
-		AggOp aggop, Rowop::Opcode opcode, RowHandle *rh, Tray *copyTray);
+		AggOp aggop, Rowop::Opcode opcode, RowHandle *rh);
 
 protected:
 	// In more complex aggregators the gadget would be of a subtype
